/*
ScareDuino - Scarey skull controlled by an Arduino.

When a switch is closed this:
- turns on two LEDs,
- starts playing a sound at a certain minimum frequency and
- starts opening the jaw, but does so in small steps.

While the jaw is still opening this:
- continues to increase the sound frequency and
- continues to open the jaw in small steps
until a certain time is up.

The jaw is left open and the sound continues playing for
a predetermined period of time.

When that time is up this:
- starts decreasing the frequency and
- starts closing the jaw in small steps
until the same time as was used for opening is up.
When the time is up this:
- turns off the sound
- turns off the LEDs.

At any time during the above process, if the switch is
opened, then the LEDs and sound are turned off and the
process of closing the jaw in steps is begun.

The motor also has a switch that the Arduino doesn't know about
that controls power to the servo. If that switch is off then the
jaw simply won't work.

NOTE: tone() causes interference with pins 3 and 11.

To see it in action, go to:
http://rimstar.org/science_electronics_projects/arduino_controlled_skull_scareduino.htm
*/
#include <Servo.h>

Servo servo;
int servoPin = 5;
int switchPin = 7;
int led1Pin = 9;
int led2Pin = 10;
int speakerPin = 12;

boolean lastSwitch = LOW;          // these are used for debouncing the switch
boolean currentSwitch = LOW;

#define JAW_DEGREES_CLOSED    0    // degrees when jaw is closed
#define JAW_DEGREES_OPENED    40   // degrees when jaw is fully open
#define JAW_TIME_STEP_SIZE    100  // time each step should take (msec)
#define JAW_DEGREES_STEP_SIZE 5    // degrees to move each step
#define JAW_TIME_TO_LEAVE_OPEN 2000 // how long to leave the jaw open (msec)
// states for the state machine below
#define JAW_OPENING 1
#define JAW_OPEN    2
#define JAW_CLOSING 3
#define JAW_CLOSED  4
int jawState;   // current state: JAW_OPENING, JAW_OPEN, JAW_CLOSING, JAW_CLOSED
int jawDegrees; // current angle of the servo
unsigned long jawStateTargetDegrees; // The angle we're trying to get the 
                                     // servo to before stopping (fully open 
                                     // or fully closed).
unsigned long jawStateTargetTime;    // The time when the next action should 
                                     // happen. Just what that action is
                                     // depends on the current state. This
                                     // prevents us doing something every single
                                     // time through the loop.
                                     // Example: While the jaw is opening, the
                                     // actions are to open the jaw a little bit
                                     // more and increase the sound frequency.

#define TONE_START_FREQ 130          // The sound frequency to start at when
                                     // starting to open jaw.
unsigned int curToneFreq;            // the current sound frequency

void setup()
{
  servo.attach(servoPin);
  pinMode(switchPin, INPUT);
  pinMode(led1Pin, OUTPUT);
  pinMode(led2Pin, OUTPUT);
  digitalWrite(led1Pin, false);
  digitalWrite(led2Pin, false);
  pinMode(speakerPin, OUTPUT);

  // when the Arduino is first turned on, close the jaw
  servo.write(JAW_DEGREES_CLOSED);
  jawDegrees = JAW_DEGREES_CLOSED;
  jawState = JAW_CLOSED;
}

boolean debounce(boolean last)
{
  boolean current = digitalRead(switchPin);
  if (last != current) {
    delay(5);
    current = digitalRead(switchPin);
  }
  return current;
}

void loop()
{
  // get the control switch's position
  currentSwitch = debounce(lastSwitch);
  if (lastSwitch != currentSwitch) {
    // the switch position changed
    lastSwitch = currentSwitch;
    if (currentSwitch == HIGH) {
      // the control was switch turned on
      digitalWrite(led1Pin, true); // turn on the LEDs
      digitalWrite(led2Pin, true);
      curToneFreq = TONE_START_FREQ; // start the sound
      tone(speakerPin, curToneFreq);
      jawState = JAW_OPENING; // start opening the jaw
      jawStateTargetTime = millis() + JAW_TIME_STEP_SIZE;
      jawStateTargetDegrees = JAW_DEGREES_OPENED;
    } else {
      // the control was switch turned off
      digitalWrite(led1Pin, false); // turn off the LEDs
      digitalWrite(led2Pin, false);
      noTone(speakerPin); // turn off the sound
      jawState = JAW_CLOSING; // start closing the jaw
      jawStateTargetTime = millis() + JAW_TIME_STEP_SIZE;
      jawStateTargetDegrees = JAW_DEGREES_CLOSED;
    }
  } else {
    switch (jawState) {
      case JAW_OPENING:
        if (millis() >= jawStateTargetTime) {
          // It's time to do some more work for opening the jaw.
          if (jawDegrees + JAW_DEGREES_STEP_SIZE < jawStateTargetDegrees) {
            // The jaw isn't fully open yet so open the jaw
            // a little more and increase the sound frequency.
            jawDegrees += JAW_DEGREES_STEP_SIZE;
            servo.write(jawDegrees);

            curToneFreq++;
            tone(speakerPin, curToneFreq);
            
            jawStateTargetTime = millis() + JAW_TIME_STEP_SIZE;
          } else {
            // We're done opening the jaw. Keep it open for a while.
            jawState = JAW_OPEN;
            jawStateTargetTime = millis() + JAW_TIME_TO_LEAVE_OPEN;
          }
        }
        break;
      case JAW_OPEN:
        // Check if it's time to start closing the jaw.
        if (millis() >= jawStateTargetTime) {
          // It's time to start closing the jaw.
          jawState = JAW_CLOSING;
          jawStateTargetTime = millis() + JAW_TIME_STEP_SIZE;
          jawStateTargetDegrees = JAW_DEGREES_CLOSED;
        }
        break;
      case JAW_CLOSING:
        if (millis() >= jawStateTargetTime) {
          // It's time to do some more work for closing the jaw.
          if (jawDegrees - JAW_DEGREES_STEP_SIZE > jawStateTargetDegrees) {
            // The jaw isn't fully closed yet so close the jaw
            // a little more and decrease the sound frequency.
            jawDegrees -= JAW_DEGREES_STEP_SIZE;
            servo.write(jawDegrees);

            curToneFreq--;
            tone(speakerPin, curToneFreq);
            
            jawStateTargetTime = millis() + JAW_TIME_STEP_SIZE;
          } else {
            // We're done closing the jaw so turn off the LEDs and the sound.
            digitalWrite(led1Pin, false);
            digitalWrite(led2Pin, false);
            noTone(speakerPin);

            jawState = JAW_CLOSED;
          }
        }
        break;
      case JAW_CLOSED:
        // we have nothing to do
        break;  
    }
  }
}

